package com.sun.corba.se.impl.encoding;

import com.sun.corba.se.impl.orbutil.ORBUtility;
import com.sun.corba.se.pept.transport.ByteBufferPool;
import java.nio.ByteBuffer;
import com.sun.corba.se.spi.orb.ORB;

public class ByteBufferWithInfo
{
    private ORB orb;
    private boolean debug;
    private int index;
    public ByteBuffer byteBuffer;
    public int buflen;
    public int needed;
    public boolean fragmented;
    
    public ByteBufferWithInfo(org.omg.CORBA.ORB orb, ByteBuffer byteBuffer, int n) {
        this.orb = (ORB)orb;
        this.debug = this.orb.transportDebugFlag;
        this.byteBuffer = byteBuffer;
        if (byteBuffer != null) {
            this.buflen = byteBuffer.limit();
        }
        this.position(n);
        this.needed = 0;
        this.fragmented = false;
    }
    
    public ByteBufferWithInfo(org.omg.CORBA.ORB orb, ByteBuffer byteBuffer) {
        this(orb, byteBuffer, 0);
    }
    
    public ByteBufferWithInfo(org.omg.CORBA.ORB orb, BufferManagerWrite bufferManagerWrite) {
        this(orb, bufferManagerWrite, true);
    }
    
    public ByteBufferWithInfo(org.omg.CORBA.ORB orb, BufferManagerWrite bufferManagerWrite, boolean b) {
        this.orb = (ORB)orb;
        this.debug = this.orb.transportDebugFlag;
        int bufferSize = bufferManagerWrite.getBufferSize();
        if (b) {
            this.byteBuffer = this.orb.getByteBufferPool().getByteBuffer(bufferSize);
            if (this.debug) {
                int identityHashCode = System.identityHashCode(this.byteBuffer);
                StringBuffer sb = new StringBuffer(80);
                sb.append("constructor (ORB, BufferManagerWrite) - got ").append("ByteBuffer id (").append(identityHashCode).append(") from ByteBufferPool.");
                this.dprint(sb.toString());
            }
        }
        else {
            this.byteBuffer = ByteBuffer.allocate(bufferSize);
        }
        this.position(0);
        this.buflen = bufferSize;
        this.byteBuffer.limit(this.buflen);
        this.needed = 0;
        this.fragmented = false;
    }
    
    public ByteBufferWithInfo(ByteBufferWithInfo byteBufferWithInfo) {
        this.orb = byteBufferWithInfo.orb;
        this.debug = byteBufferWithInfo.debug;
        this.byteBuffer = byteBufferWithInfo.byteBuffer;
        this.buflen = byteBufferWithInfo.buflen;
        this.byteBuffer.limit(this.buflen);
        this.position(byteBufferWithInfo.position());
        this.needed = byteBufferWithInfo.needed;
        this.fragmented = byteBufferWithInfo.fragmented;
    }
    
    public int getSize() {
        return this.position();
    }
    
    public int getLength() {
        return this.buflen;
    }
    
    public int position() {
        return this.index;
    }
    
    public void position(int index) {
        this.byteBuffer.position(index);
        this.index = index;
    }
    
    public void setLength(int buflen) {
        this.buflen = buflen;
        this.byteBuffer.limit(this.buflen);
    }
    
    public void growBuffer(ORB orb) {
        int buflen;
        for (buflen = this.byteBuffer.limit() * 2; this.position() + this.needed >= buflen; buflen *= 2) {}
        ByteBufferPool byteBufferPool = orb.getByteBufferPool();
        ByteBuffer byteBuffer = byteBufferPool.getByteBuffer(buflen);
        if (this.debug) {
            int identityHashCode = System.identityHashCode(byteBuffer);
            StringBuffer o = new StringBuffer(80);
            o.append("growBuffer() - got ByteBuffer id (");
            o.append(identityHashCode).append(") from ByteBufferPool.");
            this.dprint(o.toString());
        }
        this.byteBuffer.position(0);
        byteBuffer.put(this.byteBuffer);
        if (this.debug) {
            int identityHashCode2 = System.identityHashCode(this.byteBuffer);
            StringBuffer o2 = new StringBuffer(80);
            o2.append("growBuffer() - releasing ByteBuffer id (");
            o2.append(identityHashCode2).append(") to ByteBufferPool.");
            this.dprint(o2.toString());
        }
        byteBufferPool.releaseByteBuffer(this.byteBuffer);
        this.byteBuffer = byteBuffer;
        this.buflen = buflen;
        this.byteBuffer.limit(this.buflen);
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("ByteBufferWithInfo:");
        sb.append(" buflen = " + this.buflen);
        sb.append(" byteBuffer.limit = " + this.byteBuffer.limit());
        sb.append(" index = " + this.index);
        sb.append(" position = " + this.position());
        sb.append(" needed = " + this.needed);
        sb.append(" byteBuffer = " + ((this.byteBuffer == null) ? "null" : "not null"));
        sb.append(" fragmented = " + this.fragmented);
        return sb.toString();
    }
    
    protected void dprint(String s) {
        ORBUtility.dprint("ByteBufferWithInfo", s);
    }
}
